///////////////////////////////////////////////////////////////////////////////

class AwarenessPoint extends Actor
	placeable
	native;

///////////////////////////////////////////////////////////////////////////////

enum EAwarenessPointCreation
{
    kAPC_DesignerPlaced,
    kAPC_AutoGenerated,
};

///////////////////

var(Awareness) EAwarenessPointCreation creation;
var(Awareness) float minimumThreat;

var private Texture	editorTextures[EAwarenessPointCreation.EnumCount];

// If this awareness point was auto-generated, this member will contain a
// reference to the actor it was created from. If this point is designer-
// placed, this reference is left as none.
var private nocopy Actor basisActor;

// The closest navigation point to the awareness point.
// If it was autogenerated, this will be the same as the basisActor.
var private nocopy NavigationPoint closestNavigationPoint;

// Contains the static connections to other static awareness points
struct native AwarenessConnection
{
    var AwarenessPoint AwarenessPoint;
    var float Distance;
};

var private nocopy array<AwarenessConnection> connections;

///////////////////////////////////////////////////////////////////////////////

cpptext
{
    AAwarenessPoint();
    virtual void CheckForErrors();
    virtual void PostEditChange();
    virtual void PostEditLoad();
    virtual void PreEditDelete();
    virtual FVector GetAimLocation(AFiredWeapon* WeaponToAim);
    void CovertToDesignerPlaced();
    void UpdateTexture();
    void RemoveReferencesToThisPoint();
}

///////////////////////////////////////////////////////////////////////////////

event PostBeginPlay()
{
    Super.PostBeginPlay();
    Texture = editorTextures[creation];
}

///////////////////////////////////////

function Actor GetBasisActor()
{
    return basisActor;
}

function NavigationPoint GetClosestNavigationPoint()
{	
	return closestNavigationPoint;
}

///////////////////////////////////////////////////////////////////////////////

defaultproperties
{
	Texture           = Texture'SWAT4editorTex.AwarnessPointDesigner'
    editorTextures(0) = Texture'SWAT4editorTex.AwarnessPointDesigner'
    editorTextures(1) = Texture'SWAT4editorTex.AwarnessPointAuto'

    bStatic = true
	bHidden = true
	; Collision defaults should be the same as NavigationPoint
	CollisionRadius=+00030.000000
    CollisionHeight=+00080.000000

    creation = kAPC_DesignerPlaced
    minimumThreat = 0.0
}

///////////////////////////////////////////////////////////////////////////////
